library(ggplot2)
library(reshape2)
library(lattice)
library(dplyr)
library(zoo)
library(KFAS)
library(tseries)
library(timeSeries)
library(TSstudio)

path <- "../dipwells/"
file <- c("FMIB", "FMAB", "CAAB", "FMR1", "FMR2_pooled", "FOIB", "FOAB", "FOR1", "FOR2")
site <- c("RBIB", "RBAB2", "RBAB1", "RBR1", "RBR2", "BBIB", "BBAB", "BBR1", "BBR2")
location <- c("FM", "FM", "FM", "FM", "FM", "FO", "FO", "FO", "FO")
g_df <- NULL

# Zoo object to data frame helper function
zoo.to.data.frame <- function(x, index.name="Date") {
  stopifnot(is.zoo(x))
  xn <- if(is.null(dim(x))) deparse(substitute(x)) else colnames(x)
  setNames(data.frame(index(x), x, row.names=NULL), c(index.name,xn))
}


smooth <- function(ts, factor = 0.01) {
  
  # Create ts object for KFS filter
  t <- zoo_to_ts(ts)
  
  # Create model
  ssModel <- SSModel(t ~SSMtrend(1, Q = factor), H=(factor), distribution = "gaussian")
  
  # SSM fit
  ssModel <- fitSSM(ssModel, inits = c(0, 0), method = "BFGS")$model
  
  # Smooth
  kfs <- KFS(ssModel, smoothing="state", nsim=length(t))
  
  # Confidence and prediction intervals for the expected value and the observations.
  # Note that predict uses original model object, not the output from KFS.
  confSSM <- predict(ssModel, interval = "confidence", level = 0.90)
  predSSM <- predict(ssModel, interval = "prediction", level = 0.90)
  
  # Mean smooth
  tNA <- t
  tNA <- SSModel(tNA ~ SSMtrend(1, Q = list(ssModel$Q)))
  kfsNA <- KFS(tNA, "mean", "mean")
  
  # Fitted
  kfsfit <- fitted(kfsNA)
  
  # Create fitted time series
  kfs_ts <- as.zoo(kfsfit)
  
  # Create kfs data frame
  kfs_df <- zoo.to.data.frame(kfs_ts)
  
  # Return smoothed data frame
  return(kfs_df)
}

for (i in 1:length(file)) {
  
  print(file[i])
  
  # Read in precipitation (P) and discharge (WTD)
  wtdat <- read.table(paste(path, file[i], ".csv", sep=""), header = TRUE, sep = ",", as.is = TRUE)
  
  # Drop P column
  wtdat$P <- NULL
  
  # Add site field
  wtdat$Site <- site[i]
  if (i <= 5) {
    wtdat$Location <- "RB"
  }
  else {
    wtdat$Location <- "BB"
  }
  
  # Create time series of 15 min time steps
  tsWTD <- zoo(x = wtdat[, 2:3], order.by = wtdat$Date)
  
  # Copy in smoothed WTD
  wtdat$WTD <- smooth(tsWTD$WTD, factor = 0.001)[, 2]
  
  # Take out date
  dat <- subset(wtdat, select=c("Location", "Site", "WTD"))
  
  # Arrange linearly
  dat <- arrange(dat, -WTD)
  
  # Calculate percentage
  dat <- mutate(dat, WTD_pct = (percent_rank(WTD))*100)
  
  # Bind rows
  g_df <- bind_rows(dat, g_df)
}

# Set site factor
g_df$Site <- factor(g_df$Site, levels=c("RBIB", "RBAB2", "RBAB1", "RBR1", "RBR2", "BBIB", "BBAB", "BBR1", "BBR2"))

# Set location factor
g_df$Location <- factor(g_df$Location, levels=c("RB", "BB"))

# Colour blind friendly palette
cbPalette <- c("#999999", "#808000", "#E69F00", "#56B4E9", "#009E73", "#999999", "#E69F00", "#56B4E9", "#009E73")

# Plot log Q against log probability scale
p <- ggplot(g_df, aes(x=WTD_pct, y=WTD, shape = Site , colour = Site)) + 
  geom_line(size=1) +
  scale_y_reverse() +
  scale_shape_manual(name="Site", values=c(0, 1, 2, 3, 4, 5, 6, 7, 8)) +
  scale_fill_manual(name="Site", values=cbPalette) +
  scale_colour_manual(name="Site", values=cbPalette) +
  theme_light() +
  labs(x = "Percent time WTD not exceeded", y = expression('WTD (cm)')) +
  facet_wrap(facets = ~Location, labeller = labeller(Location = c("RB" = "Raised bog", "BB" = "Blanket bog"))) +
  theme_bw(base_size = 14) + theme(legend.position = "bottom", axis.text.y = element_text(hjust=0)) +
  guides(colour = guide_legend(override.aes = list(size=4)))

p

ggsave(filename="fdc_wtd5.pdf", 
       plot = p, 
       device = cairo_pdf, 
       width = 210, 
       height = 100, 
       units = "mm")








