library(hydromad)
library(zoo)
library(chron)
library(data.table)
library(hydroTSM)
library(dplyr)
library(EcoHydRology)

# Zoo object to data frame helper function
zoo.to.data.frame <- function(x, index.name="Date") {
  stopifnot(is.zoo(x))
  xn <- if(is.null(dim(x))) deparse(substitute(x)) else colnames(x)
  setNames(data.frame(index(x), x, row.names=NULL), c(index.name,xn))
}

convertFlow <- function(x, from = "L", to = "mm", area.m2) {
  
  # 1000L to a cubic meter, then divide by square m
  xn <- (x/1000) / area.m2
  
  return(xn)
}

path <- "../streamflow/"
site <- c("FMIB", "CAAB", "FMR1", "FMR2", "FOIB", "FOAB", "FOR1", "FOR2")
location <- c("FM", "FM", "FM", "FM", "FO", "FO", "FO", "FO")
type <- c("IB", "AB", "R", "R", "IB", "AB", "R", "R")
latitude <- c(56.16434123, 56.16298495, 56.13615745, 56.13863986, 58.42115029, 58.42563695, 58.4347377, 58.42507821)	
catch_area <- c(60130.688969, 6718.81952, 24608.592452, 262152.633861, 16392.379819, 50846.438335, 15805.782624, 22707.343791)
endDTS <- c(rep(as.chron("2019-11-28 00:00"), 4), as.chron("2019-09-27 00:00"), as.chron("2019-12-03 00:00"), as.chron("2019-12-03 00:00"), rep(as.chron("2019-12-03 00:00"), 2))
df.All <- data.frame()
df.Raw <- data.frame()

# Read in temperature dat
tdat <- read.table("../temp/MO_Temp.csv", header = TRUE, sep = ",", as.is = TRUE)

# Format date_times
tdat$Date <- as.chron(tdat$Date, "%d/%m/%Y")

for (i in 1:length(site)) {
  
  # Read in precipitation (P) and discharge (Q)
  pqdat <- read.table(paste(path, site[i], ".csv", sep=""), header = TRUE, sep = ",", as.is = TRUE)
  
  site[i]
  
  # Format date_times
  pqdat$Date <- as.chron(pqdat$Date, "%d/%m/%Y %H:%M")
  
  # Convert flow for L in 15 minute period
  pqdat$Q <- pqdat$Q * 60 * 15
  
  # Handle any dodgy data
  pqdat$P[pqdat$P < 0] <- NA
  pqdat$Q[pqdat$Q < 0] <- NA 
  
  # Make a copy of raw data to check our own calculations
  pqdat_raw <- pqdat
  
  # Convert catchment area to km2
  area_m2 <- catch_area[i] / 1000000
  
  # Convert discharge from L/s
  pqdat$Q <- convertFlow(pqdat$Q, from = "L", area.m2 = area_m2)
  
  # Identify any duplicates
  anyDuplicated(tdat$Date)
  
  # Create time series for PQ
  tsPQ <- zoo(x = pqdat[, 2:3], pqdat$Date)
  
  # Calculate Julian day
  tdat$Jday <- yday(tdat$Date)
  
  # Create time series to Temp
  tsT <- zoo(x = tdat[, 2:8], tdat$Date)
  
  # Initialise latitude for PET measurements
  lat_radians=latitude[i]*pi/180
  
  # Use entire PQ time-series
  tsHY <- window(tsPQ, start = as.Date("2018-07-22"), end = as.Date("2019-03-23"))
  tsT <- window(tsT, start = as.Date("2018-07-22"), end = as.Date("2019-03-23"))
  
  # Calculate approx PET from temperature
  if (location[i] == "FM") {
    PETapprox <- PET_fromTemp(Jday=tsT$Jday, Tmax_C=tsT$FM_Max, Tmin_C=tsT$FM_Min, AvgT = tsT$FM_Mean, lat_radians=lat_radians)
  }
  else {
    PETapprox <- PET_fromTemp(Jday=tsT$Jday, Tmax_C=tsT$FO_Max, Tmin_C=tsT$FO_Min, AvgT = tsT$FO_Mean, lat_radians=lat_radians)    
  }
  
  # Get any end date restrictions
  dtEnd <- endDTS[i]
  
  # If no date has been set - put end of time series
  if (is.null(endDTS[i])) {
    dtEnd = max(time(tsPQ))
  }
  
  # Get calibration window
  calib_win <- window(tsHY, start=as.Date(min(time(tsHY))), end=dtEnd)
  
  # Calculate runoff:rainfall ratio
  ok <- complete.cases(calib_win[, 1:2])
  hm_ratio <- (with(calib_win, sum(Q[ok])/sum(P[ok])))
  
  # Create time series for raw PQ
  tsPQ_raw <- zoo(x = pqdat_raw[, 2:3], pqdat_raw$Date)
  
  # Create same window for raw data - hydrological year
  win <- window(tsPQ_raw, start=as.Date("2018-07-22"), end=as.Date("2019-03-23"))
  
  # My own calculation that assumes 1mm of rainfall = 1L per sqm
  win <- na.approx(win, rule=2) # interpolate in any NAs
  
  # Calculate Q
  q <- sum(win$Q)/catch_area[i]
  
  # Calculate P
  p <- sum(win$P)
  
  # Calculate aet
  aet <- p - q
  
  # runoff/rainfall ratio
  ratio <- (q/p)
  
  # Calculate pet
  pet = sum(PETapprox)*1000
  
  # Create new row
  dfRow <- data.frame(site = site[i], ca = catch_area[i]/10000, P_mm=p, Q_mm=q, runoff_ratio=ratio*100, PET_mm=pet, AET_mm=aet, PET_AET=pet-aet)
  
  # Add new row to matrix
  df.All <- rbind(df.All, dfRow)
  
  # Create new SPSS data frame
  dfWin <- zoo.to.data.frame(win)
  
  # Calculate Q for frame (mm)
  q <- dfWin$Q/catch_area[i]
  
  # Calculate AET for frame (mm)
  aet <- dfWin$P - q
  
  # Get date
  date <- as.POSIXct(dfWin$Date, tz="GMT")
  
  # Create new data frame
  dfNew <- data.frame(Date = date, Site = site[i], ca = catch_area[i]/10000, Location = location[i], Type = type[i], P = dfWin$P, Q = q, AET=aet)
  
  # Append to raw data file for analysis
  df.Raw <- rbind(df.Raw, dfNew)
}

df.All

write.csv(df.All, "water-balance-pre-blocking-FOR2-CEDA.csv")

write.csv(df.Raw, "streamflow-P.csv")



