library(foreign)
library(ggplot2)
library(dplyr)
library(ggpubr)
library(reshape2)
library(data.table)
library(ggpubr)

# Set path
path <- "../streamflow/"

# Read in storm metrics file
dfAll <- read.table(paste(path, "flow_metrics_paper", ".csv", sep=""), header = TRUE, sep = ",", as.is = TRUE)

# Plot labels
plot_labels <- c(
  'peakQ' = 'Peak~Q~(mm~d^-1)',
  'lag' = 'Peak~lag~(h)',
  'r_lag' = 'Recession~lag~(h)',
  'inensity' = 'Hydrograph~intensity~(s^-1)',
  'bfi' = 'BFI',
  'storm_dur' = 'Stormflow~duration~(h)',
  'FM' = 'Flanders~Moss',
  'FO' = 'Forsinain'
)

# Select variables
df.no <- select(dfAll, "Site", "Location", "peakQ", "lag", "r_lag", "intensity", "bfi", "storm_dur")

# Set site factor
df.no$Site <- factor(df.no$Site, levels=c("FMIB", "CAAB", "FMR1", "FMR2", "", "FOIB", "FOAB", "FOR1", "FOR2"))

df.s <- df.no

# Format extreme outliers for peakQ
dd <- df.s %>% dplyr::filter(peakQ >= 50)
dd <- as.data.table(dd)[, list(toString(format(round(max(peakQ), 1), nsmall = 1), width=8), colnames(df.s[3]), 50), by = list(Location, Site)]

# lag
dd2 <- df.s %>% dplyr::filter(lag >= 45)
dd2 <- as.data.table(dd2)[, list(toString(format(round(max(lag), 1), nsmall = 1), width=8), colnames(df.s[4]), 45), by = list(Location, Site)]

# r_lag
dd3 <- df.s %>% dplyr::filter(r_lag >= 220)
dd3 <- as.data.table(dd3)[, list(toString(format(round(max(r_lag), 1), nsmall = 1), width=8), colnames(df.s[5]), 220), by = list(Location, Site)]

# Column names
colnames(dd) <- c("Location", "Site", "lbl", "variable", "Max")
colnames(dd2) <- c("Location", "Site", "lbl", "variable", "Max")
colnames(dd3) <- c("Location", "Site", "lbl", "variable", "Max")

df.N <- aggregate(peakQ~Site, data=df.no, FUN=function(x) c(length = length(x)))

colnames(df.N) <- c("Site", "N")

# Filter out extreme outliers
df.no <- df.no[!is.na(df.no$peakQ) & df.no$peakQ < 50,]
df.no <- df.no[!is.na(df.no$lag) & df.no$lag < 45,]
df.no <- df.no[!is.na(df.no$r_lag) & df.no$r_lag < 220,]

# Colour blind friendly palette
cbPalette <- c("#999999", "#E69F00", "#56B4E9", "#009E73", "#999999", "#E69F00", "#56B4E9", "#009E73")

# Create theme
mytheme <- theme_bw(base_size = 20) + theme(
  legend.position = "none", 
  axis.ticks.x = element_blank(), 
  axis.title.x = element_blank(),
  plot.margin = unit(c(0.2,0.3,0.2,0.3), "cm")
)

p1 <- ggplot(data = df.no, aes(x=Site, peakQ)) + geom_boxplot(aes(fill=Site), notch=TRUE) +
  scale_fill_manual(name="Site", values=cbPalette) +
  ylab(expression("Peak Q (mm d"^"-1"*")")) +
  stat_summary(fun.y="mean", color="black", shape=23) +
  scale_x_discrete(labels = c("RBIB", "RBAB1", "RBR1", "RBR2", "", "BBIB", "BBAB", "BBR1", "BBR2"), drop=FALSE) +
  geom_vline(xintercept = "") +
  geom_text(data=dd, aes(x=factor(Site), y=Max, label=lbl), size=3.5) +
  geom_segment(data=dd, aes(y=Max*0.85,yend=Max*0.95, xend=factor(Site)), arrow = arrow(length = unit(0.1,"cm"))) +
  mytheme + theme(axis.text.x = element_blank())

p2 <- ggplot(data = df.no, aes(x=Site, y=lag)) + geom_boxplot(aes(fill=Site), notch=TRUE) +
  scale_fill_manual(name="Site", values=cbPalette) +
  ylab("Peak lag (h)") +
  stat_summary(fun.y="mean", color="black", shape=23) +
  scale_x_discrete(labels = c("RBIB", "RBAB1", "RBR1", "RBR2", "", "BBIB", "BBAB", "BBR1", "BBR2"), drop=FALSE) +
  geom_vline(xintercept = "") +
  geom_text(data=dd2, aes(x=factor(Site), y=Max, label=lbl), size=3.5) +
  geom_segment(data=dd2, aes(y=Max*0.85,yend=Max*0.95, xend=factor(Site)), arrow = arrow(length = unit(0.1,"cm"))) +
  mytheme + theme(axis.text.x = element_blank())

p3 <- ggplot(data = df.no, aes(x=Site, y=r_lag)) + geom_boxplot(aes(fill=Site), notch=TRUE) +
  scale_fill_manual(name="Site", values=cbPalette) +
  ylab("Recession lag (h)") +
  stat_summary(fun.y="mean", color="black", shape=23) +
  scale_x_discrete(labels = c("RBIB", "RBAB1", "RBR1", "RBR2", "", "BBIB", "BBAB", "BBR1", "BBR2"), drop=FALSE) +
  geom_vline(xintercept = "") +
  geom_text(data=dd3, aes(x=factor(Site), y=Max, label=lbl), size=3.5) +
  geom_segment(data=dd3, aes(y=Max*0.85,yend=Max*0.95, xend=factor(Site)), arrow = arrow(length = unit(0.1,"cm"))) +
  mytheme + theme(axis.text.x = element_blank())

p4 <- ggplot(data = df.no, aes(x=Site, y=intensity)) + geom_boxplot(aes(fill=Site), notch=TRUE) +
  scale_fill_manual(name="Site", values=cbPalette) +
  ylab(expression(Intensity~(s^-1))) +
  stat_summary(fun.y="mean", color="black", shape=23) +
  scale_x_discrete(labels = c("RBIB", "RBAB1", "RBR1", "RBR2", "", "BBIB", "BBAB", "BBR1", "BBR2"), drop=FALSE) +
  geom_vline(xintercept = "") +
  mytheme + theme(axis.text.x = element_text(angle = 45, vjust = 1, hjust = 1))

p5 <- ggplot(data = df.no, aes(x=Site, y=bfi)) + geom_boxplot(aes(fill=Site), notch=TRUE) +
  scale_fill_manual(name="Site", values=cbPalette) +
  ylab("BFI") +
  stat_summary(fun.y="mean", color="black", shape=23) +
  scale_x_discrete(labels = c("RBIB", "RBAB1", "RBR1", "RBR2", "", "BBIB", "BBAB", "BBR1", "BBR2"), drop=FALSE) +
  geom_vline(xintercept = "") +
  mytheme + theme(axis.text.x = element_text(angle = 45, vjust = 1, hjust = 1))

p6 <- ggplot(data = df.no, aes(x=Site, y=storm_dur)) + geom_boxplot(aes(fill=Site), notch=TRUE) +
  scale_fill_manual(name="Site", values=cbPalette) +
  ylab("Stormflow duration (h)") +
  stat_summary(fun.y="mean", color="black", shape=23) +
  scale_x_discrete(labels = c("RBIB", "RBAB1", "RBR1", "RBR2", "", "BBIB", "BBAB", "BBR1", "BBR2"), drop=FALSE) +
  geom_vline(xintercept = "") +
  mytheme + theme(axis.text.x = element_text(angle = 45, vjust = 1, hjust = 1))

# Cow plot arrange
plots <- cowplot::plot_grid(p1, p2, p3, p4, p5, p6, nrow=2, ncol=3, labels = c('a', 'b', 'c', 'd', 'e', 'f'), label_size = 18, align = "v", rel_heights = c(0.4, 0.5))

df.N <- t(df.N)

# Format table
stable <- ggtexttable(df.N, theme = ttheme("light", base_size = 16))

fig <- cowplot::plot_grid(plots, stable, nrow=2, ncol=1, rel_heights = c(0.90, 0.10))

ggsave(filename="storm_metrics_bp_no_facets_rb_bb_rev.pdf", 
       plot = fig, 
       device = cairo_pdf, 
       width = 297, 
       height = 200, 
       units = "mm")

