library(Hmisc)
library(zoo)
library(ggplot2)
library(cowplot)
library(KFAS)
library(tseries)
library(timeSeries)
library(TSstudio)
library(dplyr)
library(lubridate)

path <- "C:\\Users\\timho\\OneDrive\\Documents\\PhD\\Data\\Hydrology\\data\\"
wtd <- c("FMIB", "CAAB", "FMR1", "FMR2")
pq <- c("FMIB", "CAAB", "FMR1", "FMR2")
catch_area <- c(60130.688969, 6718.81952, 24608.592452, 262152.633861)
site <- c("RBIB", "RBAB1", "RBR1", "RBR2")
location <- c("FM", "FM", "FM", "FM")
type <- c("IB", "AB", "R", "R")
dfFMAll <- data.frame()

# Set time zone
Sys.setenv(TZ = "GMT")

convertFlow <- function(x, from = "L", to = "mm", area.m2) {
  
  # 1000L to a cubic meter, then divide by square m
  xn <- (x/1000) / area.m2
  
  return(xn)
}

# Zoo object to data frame helper function
zoo.to.data.frame <- function(x, index.name="Date") {
  stopifnot(is.zoo(x))
  xn <- if(is.null(dim(x))) deparse(substitute(x)) else colnames(x)
  setNames(data.frame(index(x), x, row.names=NULL), c(index.name,xn))
}

for (i in 1:length(wtd)) {
  
  # Read in precipitation (P) and discharge (WTD)
  wtdat <- read.table(paste("../dipwells/", wtd[i], ".csv", sep=""), header = TRUE, sep = ",", as.is = TRUE)
  
  # Read in PQ
  pqdat <- read.table(paste("../streamflow/", pq[i], ".csv", sep=""), header = TRUE, sep = ",", as.is = TRUE)
  
  # Multiply by time step
  pqdat$Q <- pqdat$Q * 60 * 15
  
  # Convert flow to mm
  area_m2 <- catch_area[i] / 1000
  
  # Convert discharge from L/s
  pqdat$Q <- convertFlow(pqdat$Q, from = "L", area.m2 = area_m2)
  
  # Interpolate any missing values
  pqdat$Q <- na.approx(pqdat$Q, rule=2)
  
  # Format date_times
  wtdat$Date <- as.POSIXct(wtdat$Date, format="%d/%m/%Y %H:%M", tz="GMT")
  pqdat$Date <- as.POSIXct(pqdat$Date, "%d/%m/%Y %H:%M", tz="GMT")
  
  # Create time series of 15 min time steps
  tsWTD <- zoo(x = wtdat[, 2:3], order.by = wtdat$Date)
  
  # Create time series of 15 min time steps
  tsPQ <- zoo(x = pqdat[, 2:3], order.by = pqdat$Date)
  
  # Aggregate mean WTD for day
  mean_WTD <- aggregate(list(WTD = wtdat$WTD), 
                        list(Date = cut(as.Date(wtdat$Date), "days")), 
                        mean)
  
  # Aggregate Q for day
  sum_Q <- aggregate(list(Q = pqdat$Q), 
                     list(Date = cut(as.Date(pqdat$Date), "days")), 
                     sum)
  
  # Aggregate P for day
  sum_P <- aggregate(list(P = wtdat$P), 
                     list(Date = cut(as.Date(wtdat$Date), "days")), 
                     sum)
  
  # Create time series
  tsWTD <- zoo(x = mean_WTD[, 2], order.by = as.POSIXct(mean_WTD$Date, tz="GMT"))
  tsQ <- zoo(x = sum_Q[, 2], order.by = as.POSIXct(sum_Q$Date, tz="GMT"))
  tsP <- zoo(x = sum_P[, 2], order.by = as.POSIXct(sum_P$Date, tz="GMT"))
  
  # Start time
  start <- as.POSIXct("26/02/2018", "%d/%m/%Y", tz="GMT")
  end <- start + as.difftime(639, units="days")
  
  # Create empty timeseries
  seq <- seq(from=start, by=60*60*24, to=end)
  
  # Create NA data frame
  dfNA <- data.frame(Date = seq, Val=1)
  
  # Create NA time series
  tsNA <- zoo(x = dfNA[, 2:2], order.by = dfNA$Date)
  
  # Set date range
  tsWTD <- window(tsWTD, start=start, end=end)
  tsQ <- window(tsQ, start=start, end=end)
  tsP <- window(tsP, start=start, end=end)
  
  # Merge into one time-series
  tsMerge <- zoo::merge.zoo(tsNA, tsP, tsWTD, tsQ)
  
  # Convert to data frame
  df <- zoo.to.data.frame(tsMerge)
  
  # Format colnames
  colnames(df) <- c("Date", "tsNA", "P", paste(site[i], "_wtd", sep=""), paste(site[i], "_q", sep=""))
  
  # Always drop tsNA - not needed
  df$tsNA <- NULL
  
  # If not at start only include first Date column
  if (i != 1) {
    df$Date <- NULL
    df$P <- NULL
  }
  
  if (i == 1) {
    # Init FM frame
    dfFMAll <- df
  }
  else {
    # Bind FM columns
    dfFMAll <- cbind(dfFMAll, df)   
  }
}

# Convert to POSIX date format
dfFMAll$Date <- as.POSIXct(dfFMAll$Date, tz="GMT")

# Colour blind friendly palette
cbPalette <- c("#999999", "#E69F00", "#56B4E9", "#009E73", "#F0E442", "#0072B2", "#D55E00", "#CC79A7")

dfWTD <- dfFMAll
dfQ <- dfFMAll

dfWTD$RBIB_q <- NULL
dfWTD$RBAB1_q <- NULL
dfWTD$RBR1_q <- NULL
dfWTD$RBR2_q <- NULL

dfQ$RBIB_wtd <- NULL
dfQ$RBAB1_wtd <- NULL
dfQ$RBR1_wtd <- NULL
dfQ$RBR2_wtd <- NULL

colnames(dfWTD) <- c("Date", "P", "RBIB", "RBAB1", "RBR1", "RBR2")
colnames(dfQ) <- c("Time", "P", "RBIB", "RBAB1", "RBR1", "RBR2")

# Correlate
c <- cor(dfWTD[3:6], use="pairwise.complete.obs")

# Melt by site, subsite and sampling month
df.m <- reshape2::melt(dfWTD, id.vars = c("Date", "P"))

# Precipitation hyetograph
g1 <- ggplot(data=dfFMAll, aes(x = Date, y = P)) +
  geom_col(fill = "blue") +
  theme_bw(base_size = 18) +
  ylab("Total P (mm)") +
  scale_y_reverse(limits = c(42, 0)) +
  scale_x_datetime(date_breaks = "2 months") +
  ggtitle(label="(a)") +
  theme(axis.title.x    = element_blank(),
        axis.text.x     = element_blank(),
        axis.ticks.x    = element_blank(), 
        axis.title.y = element_text(size=(16)),
        plot.margin = unit(c(0, 1, -0.2, 0), "cm"))

# WTD Hydrograph
g2 <- ggplot(data = df.m, aes(x = Date, y = value, color = variable, fill = variable)) +
  geom_line(size=1) +
  ylab("Mean WTD (cm)") +
  xlab("Date") +
  scale_color_manual(name="Site", values=cbPalette) +
  scale_x_datetime(date_labels = "%b%y", date_breaks = "2 months") +
  scale_y_reverse(limits = c(72, -10)) +
  theme_bw(base_size = 18) +
  theme(legend.position = "none",
        axis.title.x    = element_blank(),
        axis.text.x     = element_blank(),
        axis.ticks.x    = element_blank(),
        axis.title.y = element_text(size=(16)),
        plot.margin = unit(c(0, 1, -0.2, 0), "cm"))

# Melt by site, subsite and sampling month
df.m2 <- reshape2::melt(dfQ, id.vars = c("Time", "P"))

# Q Hydrograph
g3 <- ggplot(data = df.m2, aes(x = Time, y = value, color = variable, fill = variable)) +
  geom_line(size=1) +
  xlab("Time") +
  ylab("Total Q (mm)") +
  scale_x_datetime(date_labels = "%b%y", date_breaks = "2 months") +
  scale_color_manual(name="Site", values=cbPalette) +
  scale_y_continuous() +
  theme_bw(base_size = 18) +
  theme(legend.position = "bottom",
        plot.margin = unit(c(0, 1, 0, 0), "cm"),
        axis.title.y = element_text(size=(16))) +
  guides(colour = guide_legend(override.aes = list(size=4)))
# Arrange plots
pp <- list(g1, g2, g3)
p <- cowplot::plot_grid(plotlist=pp, ncol=1, align='v', rel_heights = c(0.2, 0.2, 0.4))

p

ggsave(filename="stormRB.pdf", 
       plot = p, 
       device = cairo_pdf, 
       width = 297, 
       height = 210, 
       units = "mm")








